==> ToolChain_STM32_1.0.39.tar.bz: ToolChain_STM32_1.0.39/Examples/example_threading_queues.c <==
/*{ ExampleThreading::.c ************************************************
 
 Example code demonstrating how to use inter thread communication.
 
}*/

#include "example_threading_queues.h"

//{ Function definitions *************************************************

void threading_init() {
#ifdef EXTENSION_600_example_leds
    leds_initPorts();
    initPort(PB_LED1, GPIO_Mode_Out_PP);
    initPort(PB_LED2, GPIO_Mode_Out_PP);
#endif
#ifdef EXTENSION_400_lcd_320x240_olimex
    //{ init_LCD Start
    RCC_APB1PeriphClockCmd(RCC_APB1Periph_USART2, ENABLE);

    GPIO_InitTypeDef GPIO_InitStructure;
    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_All;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AIN;
    GPIO_Init(GPIOA, &GPIO_InitStructure);
    GPIO_Init(GPIOB, &GPIO_InitStructure);
    GPIO_Init(GPIOC, &GPIO_InitStructure);
    GPIO_Init(GPIOD, &GPIO_InitStructure);
    GPIO_Init(GPIOE, &GPIO_InitStructure);
    GPIO_Init(GPIOF, &GPIO_InitStructure);
    GPIO_Init(GPIOG, &GPIO_InitStructure);

    /* Enable the FSMC Clock */
    RCC_AHBPeriphClockCmd(RCC_AHBPeriph_FSMC, ENABLE);

    LCD_Init();

    //} init_LCD Ende

    LCD_SetBacklight(1);
    u16_t BgColor = LCD_RGB_TO_LCDCOLOR(200,200,255);
    LCD_Clear(BgColor);

    //LCD_DrawChar(1,1, '>', LCD_COLOR_YELLOW, LCD_COLOR_BLACK);
    LCD_ConSetColor(BgColor, LCD_COLOR_BLACK);
    LCD_ConSetPos(2,4);
    LCD_Printf("Example");
    LCD_ConSetPos(3,3);
    LCD_Printf("Threading");
#endif
}
void threading_start() {
    threading_init();

    static et_ThreadInfo_t ThreadInfo; // must be static to survive end of this function!

    // create queue Producer -> Consumer
    ThreadInfo.QueueProducer2Consumer = xQueueCreate(10, 1);

    // create queue Producer/Consumer -> Print
    ThreadInfo.QueuePrint = xQueueCreate(5, sizeof(et_PrintJob_t));

    Assert(NULL != ThreadInfo.QueueProducer2Consumer, ec_Malloc);
    Assert(NULL != ThreadInfo.QueuePrint, ec_Malloc);

    volatile signed portBASE_TYPE Task1 = xTaskCreate(taskProducer,                          // function to start as thread
                                                      (const signed char*) "taskProducer",   // thread name (just for debugging)
                                                      (size_t) 4 * configMINIMAL_STACK_SIZE, // stack size
                                                      (void *) &ThreadInfo,                  // passed as argument to taskValue()
                                                      (unsigned portBASE_TYPE) 1,            // task priority (higher values mean more process time)
                                                      (xTaskHandle*) NULL                    // can return a handle to created task
                                                      );

    volatile signed portBASE_TYPE Task2 = xTaskCreate(taskConsumer,                          // function to start as thread
                                                      (const signed char*) "taskConsumer",   // thread name (just for debugging)
                                                      (size_t) 4 * configMINIMAL_STACK_SIZE, // stack size
                                                      (void *) &ThreadInfo,                  // passed as argument to taskValue()
                                                      (unsigned portBASE_TYPE) 1,            // task priority (higher values mean more process time)
                                                      (xTaskHandle*) NULL                    // can return a handle to created task
                                                      );

    volatile signed portBASE_TYPE Task3 = xTaskCreate(taskPrint,                             // function to start as thread
                                                      (const signed char*) "taskPrint",      // thread name (just for debugging)
                                                      (size_t) 4 * configMINIMAL_STACK_SIZE, // stack size
                                                      (void *) &ThreadInfo,                  // passed as argument to taskValue()
                                                      (unsigned portBASE_TYPE) 1,            // task priority (higher values mean more process time)
                                                      (xTaskHandle*) NULL                    // can return a handle to created task
                                                      );

    Assert(0 != Task1, ec_Malloc);
    Assert(0 != Task2, ec_Malloc);
    Assert(0 != Task3, ec_Malloc);

    for (int I = 0; I < 15; I++) {
        if (I < 15)
            queuePrintJob(ThreadInfo.QueuePrint, I, 0, "%x", I);
        if (I < 13)
            queuePrintJob(ThreadInfo.QueuePrint, 0, I, "%x", I);
    }
    queuePrintJob(ThreadInfo.QueuePrint, 4, 5, "Queues", 0);

}
void taskProducer(void *pvParameters) {
    et_ThreadInfo_t* ThreadInfo = (et_ThreadInfo_t*) pvParameters;
    Assert(NULL != ThreadInfo, ec_NULL);
    Assert(NULL != ThreadInfo->QueueProducer2Consumer, ec_et_InvalidQueueGiven);

#ifdef EXTENSION_600_example_leds
    ConfigLED_t ConfigLED = { LED_STAT1, 100, 0 };

#endif
    queuePrintJob(ThreadInfo->QueuePrint, 2, 7, "Producer>", 0);

    u8_t Value = 0; //(u16_t) rand() & 0xfff;
    while (1) {
        while (xQueueSend(ThreadInfo->QueueProducer2Consumer, &Value, (portTickType) -1) != pdTRUE);

        queuePrintJob(ThreadInfo->QueuePrint, 12, 7, "%i ", Value);

        unsigned portBASE_TYPE QueueSize = uxQueueMessagesWaiting(ThreadInfo->QueueProducer2Consumer);
        queuePrintJob(ThreadInfo->QueuePrint, 6, 9, "|Q| = %i ", (u16_t) QueueSize);
#ifdef EXTENSION_600_example_leds
        for (int I = Value; I >= 0; I--) {
            portSet(ConfigLED.GPIOx, ConfigLED.Pin);
            mSleep(100);
            portClr(ConfigLED.GPIOx, ConfigLED.Pin);
            mSleep(100);
        }
#endif
        mSleep(500);

        Value++;
        if (Value == 10) Value = 0;
    }
}
void taskConsumer(void *pvParameters) {
    et_ThreadInfo_t* ThreadInfo = (et_ThreadInfo_t*) pvParameters;
    Assert(NULL != ThreadInfo, ec_NULL);
    Assert(NULL != ThreadInfo->QueueProducer2Consumer, ec_et_InvalidQueueGiven);

#ifdef EXTENSION_600_example_leds
    ConfigLED_t ConfigLED = { LED_STAT2, 100, 0 };

#endif
    queuePrintJob(ThreadInfo->QueuePrint, 2, 8, "Consumer>", 0);

    u8_t ReceivedValue = 0;
    while (1) {
        if ( xQueueReceive(ThreadInfo->QueueProducer2Consumer, &ReceivedValue,  (portTickType) -1) ) {
            if ( Assert( ReceivedValue <= 100, ec_InvalidArgument ) ) {
                queuePrintJob(ThreadInfo->QueuePrint, 12, 8, "%i ", ReceivedValue);
#ifdef EXTENSION_600_example_leds
                for (int I = ReceivedValue; I >= 0; I--) {
                    portSet(ConfigLED.GPIOx, ConfigLED.Pin);
                    mSleep(100);
                    portClr(ConfigLED.GPIOx, ConfigLED.Pin);
                    mSleep(100);
                }
#endif
                mSleep(600); // consumer is a little bit slower than producer
            }
        }
    }
}
void queuePrintJob(xQueueHandle PrintQueue, s8_t Column, s8_t Row, const char* Text, s16_t Value) {
    et_PrintJob_t PrintJob;
    PrintJob.Column = Column;
    PrintJob.Row    = Row;
    PrintJob.Value  = Value;
    strncpy(PrintJob.Text, Text, 10);

    while (xQueueSend(PrintQueue, &PrintJob, (portTickType) -1) != pdTRUE);
}
void taskPrint(void *pvParameters) {
    et_ThreadInfo_t* ThreadInfo = (et_ThreadInfo_t*) pvParameters;
    Assert(NULL != ThreadInfo, ec_InvalidArgument);

    while (1) {
        et_PrintJob_t PrintJob;
        if ( xQueueReceive(ThreadInfo->QueuePrint, &PrintJob,  (portTickType) -1) ) {
#ifdef EXTENSION_400_lcd_320x240_olimex
            if ( (PrintJob.Column != -1) && (PrintJob.Row != -1) )
                LCD_ConSetPos(PrintJob.Row, PrintJob.Column);
            LCD_Printf(PrintJob.Text, PrintJob.Value);
#endif
        }
    }
}

//} Function definitions

==> ToolChain_STM32_1.0.42.tar.bz: ToolChain_STM32_1.0.42/Examples/example_threading_queues.c <==
/*{ ExampleThreading::.c ************************************************
 
 Example code demonstrating how to use inter thread communication.
 
}*/

#include "example_threading_queues.h"

//{ Function definitions *************************************************

void threading_init() {
#ifdef EXTENSION_400_lcd_320x240_olimex
    //{ init_LCD Start
    RCC_APB1PeriphClockCmd(RCC_APB1Periph_USART2, ENABLE);

    GPIO_InitTypeDef GPIO_InitStructure;
    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_All;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AIN;
    GPIO_Init(GPIOA, &GPIO_InitStructure);
    GPIO_Init(GPIOB, &GPIO_InitStructure);
    GPIO_Init(GPIOC, &GPIO_InitStructure);
    GPIO_Init(GPIOD, &GPIO_InitStructure);
    GPIO_Init(GPIOE, &GPIO_InitStructure);
    GPIO_Init(GPIOF, &GPIO_InitStructure);
    GPIO_Init(GPIOG, &GPIO_InitStructure);

    /* Enable the FSMC Clock */
    RCC_AHBPeriphClockCmd(RCC_AHBPeriph_FSMC, ENABLE);

    LCD_Init();

    //} init_LCD Ende

    LCD_SetBacklight(1);
    u16_t BgColor = LCD_RGB_TO_LCDCOLOR(200,200,255);
    LCD_Clear(BgColor);

    //LCD_DrawChar(1,1, '>', LCD_COLOR_YELLOW, LCD_COLOR_BLACK);
    LCD_ConSetColor(BgColor, LCD_COLOR_BLACK);
    LCD_ConSetPos(2,4);
    LCD_Printf("Example");
    LCD_ConSetPos(3,3);
    LCD_Printf("Threading");
#endif
}
void threading_start() {
    threading_init();

    static et_ThreadInfo_t ThreadInfo; // must be static to survive end of this function!

    // create queue Producer -> Consumer
    ThreadInfo.QueueProducer2Consumer = xQueueCreate(10, 1);

    // create queue Producer/Consumer -> Print
    ThreadInfo.QueuePrint = xQueueCreate(5, sizeof(et_PrintJob_t));

    Assert(NULL != ThreadInfo.QueueProducer2Consumer, ec_Malloc);
    Assert(NULL != ThreadInfo.QueuePrint, ec_Malloc);

    volatile signed portBASE_TYPE Task1 = xTaskCreate(taskProducer,                          // function to start as thread
                                                      (const signed char*) "taskProducer",   // thread name (just for debugging)
                                                      (size_t) 4 * configMINIMAL_STACK_SIZE, // stack size
                                                      (void *) &ThreadInfo,                  // passed as argument to taskValue()
                                                      (unsigned portBASE_TYPE) 1,            // task priority (higher values mean more process time)
                                                      (xTaskHandle*) NULL                    // can return a handle to created task
                                                      );

    volatile signed portBASE_TYPE Task2 = xTaskCreate(taskConsumer,                          // function to start as thread
                                                      (const signed char*) "taskConsumer",   // thread name (just for debugging)
                                                      (size_t) 4 * configMINIMAL_STACK_SIZE, // stack size
                                                      (void *) &ThreadInfo,                  // passed as argument to taskValue()
                                                      (unsigned portBASE_TYPE) 1,            // task priority (higher values mean more process time)
                                                      (xTaskHandle*) NULL                    // can return a handle to created task
                                                      );

    volatile signed portBASE_TYPE Task3 = xTaskCreate(taskPrint,                             // function to start as thread
                                                      (const signed char*) "taskPrint",      // thread name (just for debugging)
                                                      (size_t) 4 * configMINIMAL_STACK_SIZE, // stack size
                                                      (void *) &ThreadInfo,                  // passed as argument to taskValue()
                                                      (unsigned portBASE_TYPE) 1,            // task priority (higher values mean more process time)
                                                      (xTaskHandle*) NULL                    // can return a handle to created task
                                                      );

    Assert(0 != Task1, ec_Malloc);
    Assert(0 != Task2, ec_Malloc);
    Assert(0 != Task3, ec_Malloc);

    for (int I = 0; I < 15; I++) {
        if (I < 15)
            queuePrintJob(ThreadInfo.QueuePrint, I, 0, "%x", I);
        if (I < 13)
            queuePrintJob(ThreadInfo.QueuePrint, 0, I, "%x", I);
    }
    queuePrintJob(ThreadInfo.QueuePrint, 4, 5, "Queues", 0);

}
void taskProducer(void *pvParameters) {
    et_ThreadInfo_t* ThreadInfo = (et_ThreadInfo_t*) pvParameters;
    Assert(NULL != ThreadInfo, ec_NULL);
    Assert(NULL != ThreadInfo->QueueProducer2Consumer, ec_InvalidArgument);

    queuePrintJob(ThreadInfo->QueuePrint, 2, 7, "Producer>", 0);

#ifdef TTC_LED1
    ttc_Port_t Port_LED1;
    ttc_initPort(&Port_LED1, TTC_LED1, tgm_output_push_pull);
#endif

    u8_t Value = 0; //(u16_t) rand() & 0xfff;
    while (1) {
        while (xQueueSend(ThreadInfo->QueueProducer2Consumer, &Value, (portTickType) -1) != pdTRUE);

        queuePrintJob(ThreadInfo->QueuePrint, 12, 7, "%i ", Value);

        unsigned portBASE_TYPE QueueSize = uxQueueMessagesWaiting(ThreadInfo->QueueProducer2Consumer);
        queuePrintJob(ThreadInfo->QueuePrint, 6, 9, "|Q| = %i ", (u16_t) QueueSize);
#ifdef TTC_LED1
        for (int I = Value; I >= 0; I--) {
            ttc_portSet(&Port_LED1);
            mSleep(100);
            ttc_portClr(&Port_LED1);
            mSleep(100);
        }
#endif
        mSleep(500);

        Value++;
        if (Value == 10) Value = 0;
    }
}
void taskConsumer(void *pvParameters) {
    et_ThreadInfo_t* ThreadInfo = (et_ThreadInfo_t*) pvParameters;
    Assert(NULL != ThreadInfo, ec_NULL);
    Assert(NULL != ThreadInfo->QueueProducer2Consumer, ec_InvalidArgument);

    queuePrintJob(ThreadInfo->QueuePrint, 2, 8, "Consumer>", 0);
#ifdef TTC_LED2
    ttc_Port_t Port_LED2;
    ttc_initPort(&Port_LED2, TTC_LED2, tgm_output_push_pull);
#endif

    u8_t ReceivedValue = 0;
    while (1) {
        if ( xQueueReceive(ThreadInfo->QueueProducer2Consumer, &ReceivedValue,  (portTickType) -1) ) {
            if ( Assert( ReceivedValue <= 100, ec_InvalidArgument ) ) {
                queuePrintJob(ThreadInfo->QueuePrint, 12, 8, "%i ", ReceivedValue);
#ifdef TTC_LED2
                for (int I = ReceivedValue; I >= 0; I--) {
                    ttc_portSet(&Port_LED2);
                    mSleep(100);
                    ttc_portClr(&Port_LED2);
                    mSleep(100);
                }
#endif
                mSleep(600); // consumer is a little bit slower than producer
            }
        }
    }
}
void queuePrintJob(xQueueHandle PrintQueue, s8_t Column, s8_t Row, const char* Text, s16_t Value) {
    et_PrintJob_t PrintJob;
    PrintJob.Column = Column;
    PrintJob.Row    = Row;
    PrintJob.Value  = Value;
    strncpy(PrintJob.Text, Text, 10);

    while (xQueueSend(PrintQueue, &PrintJob, (portTickType) -1) != pdTRUE);
}
void taskPrint(void *pvParameters) {
    et_ThreadInfo_t* ThreadInfo = (et_ThreadInfo_t*) pvParameters;
    Assert(NULL != ThreadInfo, ec_InvalidArgument);

    while (1) {
        et_PrintJob_t PrintJob;
        if ( xQueueReceive(ThreadInfo->QueuePrint, &PrintJob,  (portTickType) -1) ) {
#ifdef EXTENSION_400_lcd_320x240_olimex
            if ( (PrintJob.Column != -1) && (PrintJob.Row != -1) )
                LCD_ConSetPos(PrintJob.Row, PrintJob.Column);
            LCD_Printf(PrintJob.Text, PrintJob.Value);
#endif
        }
    }
}

//} Function definitions
